import json

from typing import Any, Callable, Dict, List, Optional, Union

from langchain.docstore.document import Document
from langchain.document_loaders.base import BaseLoader
from langchain.vectorstores import Pinecone

class BerryDBJSONCSVLoader(BaseLoader):
  """Loads a JSON file using a jq schema.

  Example:
      [{"text": ...}, {"text": ...}, {"text": ...}] -> schema = .[].text
      {"key": [{"text": ...}, {"text": ...}, {"text": ...}]} -> schema = .key[].text
      ["", "", ""] -> schema = .[]
  """

  def __init__(
    self,
    json_data: Union[List, Dict],
    #jq_schema: str,
    content_key: Optional[str] = None,
    metadata_func: Optional[Callable[[Dict, Dict], Dict]] = None,
    text_content: bool = True
  ):
    """Initialize the JSONLoader.

    Args:
        json_data (Union[List, Dict]): JSON data to be parsed.
        jq_schema (str): The jq schema to use to extract the data or text from
            the JSON.
        content_key (str): The key to use to extract the content from the JSON if
            the jq_schema results to a list of objects (dict).
        metadata_func (Callable[Dict, Dict]): A function that takes in the JSON
            object extracted by the jq_schema and the default metadata and returns
            a dict of the updated metadata.
        text_content (bool): Boolean flag to indicate whether the content is in
            string format, default to True.
    """
    # try:
        #import jq  # noqa:F401
    # except ImportError:
    #     raise ImportError(
    #         "jq package not found, please install it with `pip install jq`"
    #     )
    self._json_data = json_data
    #self._jq_schema = jq.compile(jq_schema)
    self._content_key = content_key
    self._metadata_func = metadata_func
    self._text_content = text_content

  def load(self) -> List[Document]:
    """Load and return documents from the JSON file."""
    #pdb.set_trace()
    docs: List[Document] = []
    self._parse(self._json_data, docs)
    return docs

  def _parse(self, content: str, docs: List[Document]) -> None:
    """Convert given content to documents."""
    data = content #self._jq_schema.input(content)
    page_content = ''
    # Perform some validation
    # This is not a perfect validation, but it should catch most cases
    # and prevent the user from getting a cryptic error later on.
    if self._content_key is not None:
      self._validate_content_key(data)

    #data = data.all()
    for i, sample in enumerate(data, len(docs) + 1):
      metadata = dict(
          source='json input',
          row=i,
      )
      hyphen = '-'
      text = self._get_text(sample=sample, metadata=metadata)
      for k, v in text.items():
          page_content = "\n".join(f"{k.strip()}: {str(v).strip()}" for k, v in text.items())
      print(f'\n{hyphen *  10}\n\n {page_content}')
      docs.append(Document(page_content=page_content, metadata=metadata))

  def _get_text(self, sample: Any, metadata: dict) -> str:
    """Convert sample to string format"""
    if self._content_key is not None:
      content = sample.get(self._content_key)
      if self._metadata_func is not None:
          # We pass in the metadata dict to the metadata_func
          # so that the user can customize the default metadata
          # based on the content of the JSON object.
          metadata = self._metadata_func(sample, metadata)
    else:
      content = sample

    if self._text_content and not isinstance(content, str):
      raise ValueError(
          f"Since Text_content is true, the Expected page_content is string, got {type(content)} instead. \
              Set `text_content=False` if the desired input for \
              `page_content` is not a string \
              The loader expects the sent in content to resolve to a Dict object eventually"
      )

    # In case the text is None, set it to an empty string
    elif isinstance(content, str):
      content = json.loads(str)
      if not isinstance(content, dict):
        raise ValueError(
          f"Unable to convert Text Content to a Dict object, converted type from Text is {type(content)}. \
              Set `text_content=False` if the desired input for \
              `page_content` is not a string \
              The loader expects the sent in content to resolve to a Dict object eventually"
      )
      return content
    elif isinstance(content, dict):
      return content
    else:
      raise ValueError(
          f"The loader expects the sent in content to resolve to a Dict object eventually \
            If you are passing a string as content, make sure that: \
            1. Text_content is set to true \
            2. The passed in string(text content) can resolve to a Dict object"
      )

  def _validate_content_key(self, data: Any) -> None:
    """Check if a content key is valid"""
    sample = data.first()
    if not isinstance(sample, dict):
      raise ValueError(
          f"Expected the jq schema to result in a list of objects (dict), \
              so sample must be a dict but got `{type(sample)}`"
      )

    if sample.get(self._content_key) is None:
      raise ValueError(
          f"Expected the jq schema to result in a list of objects (dict) \
              with the key `{self._content_key}`"
      )

    if self._metadata_func is not None:
      sample_metadata = self._metadata_func(sample, {})
      if not isinstance(sample_metadata, dict):
        raise ValueError(
            f"Expected the metadata_func to return a dict but got \
                `{type(sample_metadata)}`"
        )



import requests
from requests import Response
import json
import logging
from urllib.parse import quote
from utils.utils import Utils
from constants.constants import debug_mode
from constants.constants import (
    documents_url,
    query_url,
    document_by_id_url,
    bulk_upsert_documents_url,
    transcription_url,
    transcription_yt_url,
    caption_url,
    create_label_studio_project_url,
    import_label_studio_project_url,
    reimport_label_studio_project_url,
    couchbase_config,
    SEMANTICS_PREDICT_URL,
    SEMANTICS_ANNOTATE_URL,
    LOGGING_LEVEL,
    NER_SE_TYPE,
    MEDICAL_NER_SE_TYPE,
    TEXT_CLASSIFICATION_SE_TYPE,
    TEXT_SUMMARIZATION_SE_TYPE,
    IMAGE_CLASSIFICATION_SE_TYPE,
    IMAGE_CAPTIONING_SE_TYPE,
    PNEUMONIA_SE_TYPE,
    ALZHEIMER_SE_TYPE,
    FASHION_SE_TYPE,
    AUDIO_TRANSCRIPTION_SE_TYPE,
    extract_pdf_url,
    chat_with_database_url,
    embed_database_url,
)
from unstructured.documents.elements import PageBreak
from unstructured.partition.pdf import partition_pdf
from unstructured.chunking.title import chunk_by_title

logging.basicConfig(level=LOGGING_LEVEL, format="%(asctime)s - %(levelname)s - %(message)s")


logger = logging.getLogger(__name__)


class Database:
    __api_key: str
    __bucket_name: str
    __database_id: int
    __database_name: str

    def __init__(self, api_key: str, bucket_name: str, database_id: int, database_name: str):
        if api_key is None:
            Utils.print_error_and_exit("API Key cannot be None")
        if bucket_name is None:
            Utils.print_error_and_exit("Bucket name cannot be None")
        if database_id is None:
            Utils.print_error_and_exit("Database not found")
        self.__api_key = api_key
        self.__bucket_name = bucket_name
        self.__database_id = database_id
        self.__database_name = database_name

    def apiKey(self):
        return self.__api_key

    def bucketName(self):
        return self.__bucket_name

    def databaseId(self):
        """Function summary

        Args:
                No Arguments

        Returns:
                str: Get the database ID of the connected database
        """
        return self.__database_id

    def databaseName(self):
        """Function summary

        Args:
                No Arguments

        Returns:
                str: Get the database name of the connected database
        """
        return self.__database_name

    def get_all_documents(self):
        """Function summary

        Args:
                No Arguments

        Returns:
                list: Return a list of documents in the connected database
        """

        url = documents_url
        params = {
            "apiKey": self.__api_key,
            "bucket": self.__bucket_name,
            "databaseId": self.__database_id,
        }

        if debug_mode:
            print("url:", url)
            print("params:", params)

        try:
            response = requests.get(url, params=params)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("documents result ", response.json())
            return json.loads(response.text)
        except Exception as e:
            print("Failed to fetch document: {}".format(str(e)))
            return []

    def get_all_documents_with_col_filter(self, col_filter=["*"]):
        """Function summary

        Args:
                arg1 (list<str>): Column list (Optional)

        Returns:
                list: Return a list of filtered documents in the connected database
        """

        url = documents_url
        """ params = {
			"columns": col_filter,
			"apiKey": self.__api_key,
			"bucket": self.__bucket_name,
			"databaseId": self.__database_id,
		} """
        url += "?apiKey=" + self.__api_key
        url += "&bucket=" + self.__bucket_name
        url += "&databaseId=" + str(self.__database_id)
        url += "&columns=" + (",".join(col_filter))

        if debug_mode:
            print("url:", url)
        try:
            response = requests.get(url)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("documents result ", response.json())
            # return response.json()
            return json.loads(response.text)
        except Exception as e:
            print("Failed to fetch document: {}".format(str(e)))
            return []

    def get_document_by_object_id(
        self,
        document_id,
        key_name=None,
        key_value=None,
    ):
        """Function summary

        Args:
                arg1 (str): Document Key/Id
                arg2 (str): Key Name (optional)
                arg3 (str): Key Value (optional)

        Returns:
                list: List of Documents matching the document ID in the connected database
        """

        url = document_by_id_url.format(quote(document_id))
        params = {
            "apiKey": self.__api_key,
            "bucket": self.__bucket_name,
            "databaseId": self.__database_id,
        }

        if document_id is not None:
            params["docId"] = document_id
        if key_name is not None:
            params["keyName"] = key_name
        if key_value is not None:
            params["keyValue"] = key_value

        if debug_mode:
            print("url:", url)
            print("params:", params)

        try:
            response = requests.get(url, params=params)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.text, response.status_code)
            jsonRes = response.json()
            if debug_mode:
                print("docById result ", jsonRes)
            return jsonRes
        except Exception as e:
            print("Failed to fetch document by id {} : {}".format(document_id, str(e)))
            return ""

    def query(self, query: str):
        """Function summary

        Args:
                arg1 (str): Query String

        Returns:
                list: Return a list of documents that match the query on the connected database.
        """

        url = query_url
        params = {
            "apiKey": self.__api_key,
            "bucket": self.__bucket_name,
            "databaseId": self.__database_id,
        }
        payload = query

        if debug_mode:
            print("url:", url)
            print("query:", query)
            print("params:", params)
        headers = Utils.get_headers(self.__api_key)

        try:
            response = requests.post(url, data=payload, headers=headers, params=params)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("query result ", response.json())
            return json.loads(response.text)
        except Exception as e:
            print("Failed to query : {}".format(str(e)))
            return ""

    def __upsert(self, documents) -> str:
        url = bulk_upsert_documents_url
        params = {
            "apiKey": self.__api_key,
            "bucket": self.__bucket_name,
            "databaseId": self.__database_id,
        }

        payload = json.dumps(documents)
        if debug_mode:
            print("url:", url)
            print("payload:", payload)
        headers = Utils.get_headers(self.__api_key)

        try:
            response = requests.post(url, data=payload, headers=headers, params=params)
            if response.status_code != 200:
                try:
                    resp_content = response.json()
                except ValueError:
                    resp_content = response.text
                Utils.handleApiCallFailure(resp_content, response.status_code)
            if debug_mode:
                print("upsert result ", response)
            return response.text
        except Exception as e:
            print("Failed to upsert document: {}".format(str(e)))
            return ""

    def upsert(self, documents) -> str:
        """Function summary

        Args:
                arg1 (str): List of documents Object to add/update (Each document should have a key 'id' else a random string is assigned)

        Returns:
                str: Success/Failure message
        """
        try:
            if type(documents) != list:
                documents = [documents]
            return self.__upsert(documents)
        except Exception as e:
            print("Failed to upsert documents: {}".format(str(e)))
            return ""

    def upsert_document(self, documents) -> str:
        """(DEPRECATED)
        Function summary

        Args:
                arg1 (str): List of documents Object to add/update (Each document should have a key 'id' else a random string is assigned)

        Returns:
                str:  Success/Failure message
        """

        print("upsert_document is deprecated, please use upsert instead.")
        return ""

    def deleteDocument(self, document_id):
        """Function summary

        Args:
                arg1 (str): Document ID

        Returns:
                str: Success/Failure message
        """

        url = document_by_id_url.format(quote(document_id))
        params = {
            "apiKey": self.__api_key,
            "bucket": self.__bucket_name,
            "databaseId": self.__database_id,
        }

        if debug_mode:
            print("url:", url)
            print("params:", params)

        try:
            response = requests.delete(url, params=params)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            jsonRes = response.text
            if debug_mode:
                print("Delete document result ", jsonRes)
            return jsonRes
        except Exception as e:
            print("Failed to delete document by id {}, reason : {}".format(document_id, str(e)))
            return ""

    def transcribe(self, video_url: str):
        url = transcription_url

        body = {
            "url": video_url,
        }

        payload = json.dumps(body)
        if debug_mode:
            print("url:", url)
            print("payload:", payload)
        headers = Utils.get_headers(self.__api_key)

        try:
            response = requests.post(url, headers=headers, data=payload)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            res = response.text
            if debug_mode:
                print("Transcription result: ", res)
            return res
        except Exception as e:
            print(f"Failed to get transcription for the url {video_url}, reason : {str(e)}")
            return ""

    def transcribeYT(self, video_url: str):

        url = transcription_yt_url

        body = {
            "url": video_url,
        }

        payload = json.dumps(body)
        if debug_mode:
            print("url:", url)
            print("payload:", payload)
        headers = Utils.get_headers(self.__api_key)

        try:
            response = requests.post(url, headers=headers, data=payload)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            res = response.text
            if debug_mode:
                print("Youtube transcription result: ", res)
            return res
        except Exception as e:
            print(f"Failed to get transcription for the youtube url {video_url}, reason : {str(e)}")
            return ""

    def caption(self, image_url: str):
        url = caption_url

        body = {
            "url": image_url,
        }

        payload = json.dumps(body)
        if debug_mode:
            print("url:", url)
            print("payload:", payload)
        headers = Utils.get_headers(self.__api_key)

        try:
            response = requests.post(url, headers=headers, data=payload)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            res = response.text
            if debug_mode:
                print("Caption result: ", res)
            return res
        except Exception as e:
            print(f"Failed to get caption for the url {image_url}, reason : {str(e)}")
            return ""

    def embed(
        self,
        open_ai_api_key,
        embedding_function=None,
        limit=None,
        tokens_per_minute=None,
        model=None,
        system_prompt=None,
        compute_locally=False,
    ):
        """Function summary

        Args:
                arg1 (str): OpenAI API key to embed the database

        Returns:
                str: Success/error message of embedding the database
        """

        if compute_locally is True:
            from embeddings.embeddings import Embeddings

            embeddings = Embeddings(
                self.__database_id,
                self.__database_name,
                self.__bucket_name,
                self.__api_key,
                open_ai_api_key,
                limit,
                embedding_function,
                tokens_per_minute,
                model,
                system_prompt,
            )
            return embeddings.embedDb(compute_locally)

        url = embed_database_url

        body = {
            "database": self.__database_name,
            "apiKey": self.__api_key,
            "openAIApiKey": open_ai_api_key,
        }

        payload = json.dumps(body)
        if debug_mode:
            print("url:", url)
            print("payload:", payload)
        headers = Utils.get_headers(self.__api_key)

        try:
            response = requests.post(url, headers=headers, data=payload)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            res = response.json()
            if debug_mode:
                print("Embed result: ", res)
            return res
        except Exception as e:
            errMsg = "Failed to embed the database. Please try again later."
            print(f"{errMsg}, reason : {str(e)}")
            return

    def chat(
        self,
        open_ai_api_key,
        question,
        k=None,
        temperature=None,
        model=None,
        system_prompt=None,
        session_id=None,
        compute_locally=False,
    ):
        """Function summary

        Args:
                arg1 (str): OpenAI API key to embed the database
                arg2 (str): Query/Question to for your database
                arg3 (str): K value (optional)
                arg4 (str): Temperature for the LLM (optional)
                arg5 (str): Model Name (optional)
                arg6 (str): System prompt (optional)
                arg7 (bool): Chat with DB locally (optional)

        Returns:
                str: Answer/error to the query
        """

        from embeddings.embeddings import Embeddings

        if compute_locally is True:
            embeddings = Embeddings(
                self.__database_id,
                self.__database_name,
                self.__bucket_name,
                self.__api_key,
                open_ai_api_key,
                None,
                None,
                None,
                model,
                system_prompt,
                session_id,
            )
            return embeddings.ask_and_get_answers(question, k, temperature, compute_locally)

        url = chat_with_database_url

        body = {
            key: value
            for key, value in {
                "question": question,
                "k": k,
                "database": self.__database_name,
                "apiKey": self.__api_key,
                "openAIApiKey": open_ai_api_key,
                "model": model,
                "systemPrompt": system_prompt,
                "temperature": temperature,
            }.items()
            if value is not None
        }

        payload = json.dumps(body)
        if debug_mode:
            print("url:", url)
            print("payload:", payload)
        headers = Utils.get_headers(self.__api_key)

        try:
            response = requests.post(url, headers=headers, data=payload)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            res = response.json()
            if debug_mode:
                print("Database chat result: ", res)
            return res['answer']
        except Exception as e:
            errMsg = "Failed to chat with the database. Please try again later."
            print(f"{errMsg}, reason : {str(e)}")
            return

    def ask_and_get_answers(
        self,
        open_ai_api_key,
        question,
        k=None,
        temperature=None,
        model=None,
        system_prompt=None,
        session_id=None,
        compute_locally=False,
    ):
        """Function summary

        Args:
                arg1 (str): OpenAI API key to embed the database
                arg2 (str): Query/Question to for your database
                arg3 (str): K value (optional)
                arg4 (str): Temperature for the LLM (optional)
                arg5 (str): Model Name (optional)
                arg6 (str): System prompt (optional)
                arg7 (bool): Chat with DB locally (optional)

        Returns:
                str: Answer/error to the query
        """
        print('WARNING: This is method is deprecated. Use chat(open_ai_api_key, question) instead.')
        from embeddings.embeddings import Embeddings

        if compute_locally is True:
            embeddings = Embeddings(
                self.__database_id,
                self.__database_name,
                self.__bucket_name,
                self.__api_key,
                open_ai_api_key,
                None,
                None,
                None,
                model,
                system_prompt,
                session_id,
            )
            return embeddings.ask_and_get_answers(question, k, temperature, compute_locally)

        url = chat_with_database_url

        body = {
            key: value
            for key, value in {
                "question": question,
                "k": k,
                "database": self.__database_name,
                "apiKey": self.__api_key,
                "openAIApiKey": open_ai_api_key,
                "model": model,
                "systemPrompt": system_prompt,
                "temperature": temperature,
            }.items()
            if value is not None
        }

        payload = json.dumps(body)
        if debug_mode:
            print("url:", url)
            print("payload:", payload)
        headers = Utils.get_headers(self.__api_key)

        try:
            response = requests.post(url, headers=headers, data=payload)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            res = response.json()
            if debug_mode:
                print("Database chat result: ", res)
            return res['answer']
        except Exception as e:
            errMsg = "Failed to chat with the database. Please try again later."
            print(f"{errMsg}, reason : {str(e)}")
            return

    def ingest_pdf(self, file_list, extract_json_path=None, compute_locally=True):
        """Ingests a list of PDF files and extracts their content.

        Args:
            file_list (list[File]): List of PDF files to be processed.
            extract_json_path (str, optional): Path to save the extracted data in JSON format. Defaults to None, which saves the data under 'content'.
            compute_locally (bool, optional): If True, the processing is done locally; otherwise, it is done on BerryDB servers. Defaults to True.

        Returns:
            If compute_locally is True, returns a list of extracted documents. Otherwise, returns a success/failure message.
        """
        try:
            if type(file_list) is str:
                file_list = [file_list]
            if not file_list or not len(file_list):
                raise ValueError("At least one file must be provided")
            if len(file_list) > 5:
                raise ValueError("Exceeded maximum allowed files (5)")

            for file in file_list:
                if not file.endswith(".pdf"):
                    raise ValueError("All files must be of type PDF")

            extract_json_path = extract_json_path or "content"

            Utils.validate_json_path(extract_json_path)

            files = []
            compiled_extracted_docs = []
            for file_path in file_list:
                files.append(("files", open(file_path, "rb")))
                filename = file_path.split('/')[-1]
                if compute_locally is not True:
                    compute_locally = False

                if compute_locally:
                    print(f"Processing file: {filename}")
                    pdf_elements = partition_pdf(
                        file_path,
                        url=None,
                        strategy="hi_res",
                        infer_table_structure=True,
                    )

                    # Clean the elements
                    cleaned_elements = []
                    for el in pdf_elements:
                        if el.category != "PageBreak" or el.category != "Header" or el.category != "Footer":
                            cleaned_elements.append(el)

                    # Chunk the elements by title
                    chunks = chunk_by_title(
                        cleaned_elements,
                        max_characters=2000,
                        combine_text_under_n_chars=0,
                    )

                    # Convert the chunk into documents
                    extracted_docs = []
                    for chunk in chunks:
                        chunk_dict = chunk.to_dict()
                        content_type = chunk_dict["type"] if "type" in chunk_dict else ""
                        content_text = str(chunk_dict["text"]) if "text" in chunk_dict else ""
                        doc = {}
                        doc['type'] = content_type
                        doc['bbox'] = {}
                        doc['text'] = content_text
                        doc['source'] = filename
                        extracted_doc = Utils.create_nested_dict(extract_json_path, doc)
                        extracted_docs.append(extracted_doc)
                    if debug_mode:
                        print("Extracted Documents: ", extracted_docs)
                    print(f"Processing complete: {filename}")
                    self.upsert(extracted_docs)
                    compiled_extracted_docs += extracted_docs

                if compute_locally:
                    return compiled_extracted_docs

            url = extract_pdf_url

            params = {
                "databaseId": self.__database_id,
                "databaseName": self.__database_name,
                "apiKey": self.__api_key,
                "extractJsonPath": extract_json_path,
            }

            if debug_mode:
                print("url:", url)
                print("params:", params)

            response = requests.post(url, files=files, params=params)

            if response.status_code == 200:
                print("Success")
                response_json = response.json()
                if response_json["success"]:
                    return response_json["message"]
            else:
                print(f"Failed with ingest PDFs, status code: {response.status_code}")
        except Exception as e:
            print(f"Failed with ingest PDFs, reason: {e}")
            if debug_mode:
                raise e

    # Sematic Extraction methods
    def ner(self, json_path, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): document IDs of the documents you want to extract the data of (optional)
                arg3 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = NER_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def medical_ner(self, json_path, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): document IDs of the documents you want to extract the data of (optional)
                arg3 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = MEDICAL_NER_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def text_summarization(self, json_path, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): document IDs of the documents you want to extract the data of (optional)
                arg3 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = TEXT_SUMMARIZATION_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def image_classification(self, json_path, labels, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): Labels for the classification of text
                arg3 (str): document IDs of the documents you want to extract the data of (optional)
                arg4 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = IMAGE_CLASSIFICATION_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, labels, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def image_captioning(self, json_path, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): document IDs of the documents you want to extract the data of (optional)
                arg3 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = IMAGE_CAPTIONING_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def pneumonia_detection(self, json_path, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): document IDs of the documents you want to extract the data of (optional)
                arg3 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = PNEUMONIA_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def alzheimer_detection(self, json_path, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): document IDs of the documents you want to extract the data of (optional)
                arg3 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = ALZHEIMER_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def fashion(self, json_path, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): document IDs of the documents you want to extract the data of (optional)
                arg3 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = FASHION_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def audio_transcription(self, json_path, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): document IDs of the documents you want to extract the data of (optional)
                arg3 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = AUDIO_TRANSCRIPTION_SE_TYPE
        try:
            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def text_classification(self, json_path, labels, document_ids=[], annotate=False):
        """Function summary

        Args:
                arg1 (str): JSON path to the object you want to extract semantic for
                arg2 (str): Labels for the classification of text
                arg3 (str): document IDs of the documents you want to extract the data of (optional)
                arg4 (str): Add sematic data to the document (optional)

        Returns:
                list | str: sematic data | hash
        """
        extraction_type = TEXT_CLASSIFICATION_SE_TYPE
        if not (labels and len(labels)):
            raise ValueError(f"Labels are required for {extraction_type} to classify the text.")
        try:

            return self.__semantic_extraction_base(extraction_type, json_path, document_ids, labels, annotate)

        except Exception as e:
            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
            raise e

    def __semantic_extraction_base(self, extraction_type, json_path, document_ids=None, labels=None, annotate=False):

        if not json_path:
            raise ValueError("JSON path is required")
        if not annotate and not (document_ids and len(document_ids)):
            raise ValueError("Document IDs are required if you are not annotating the document")

        url = SEMANTICS_PREDICT_URL
        if annotate:
            url = SEMANTICS_ANNOTATE_URL

        params = {
            "apiKey": self.__api_key,
        }

        body = {
            "databaseId": self.__database_id,
            "documentIds": document_ids,
            "extract": extraction_type,
            "jsonPath": json_path,
        }

        if labels and len(labels):
            body["labels"] = labels

        payload = json.dumps(body)
        headers = Utils.get_headers(self.__api_key)

        logger.debug("url:", url)
        logger.debug("params:", params)
        logger.debug("payload:", payload)
        logger.debug("headers:", headers)

        response: Response = requests.post(url, params=params, data=payload, headers=headers)

        if response.status_code == 200:
            return response.json()

        logger.error("response status code: ", response.status_code)
        logger.error("response body: ", response.json())



import os
import tiktoken
import faiss
import time
import numpy as np
from langchain.vectorstores.faiss import FAISS
from langchain_openai import ChatOpenAI
from langchain.chains import RetrievalQA
from langchain.docstore.document import Document
from langchain.docstore.in_memory import InMemoryDocstore
from langchain.prompts import PromptTemplate
from langchain_community.embeddings import OpenAIEmbeddings
from langchain_community.llms import HuggingFaceEndpoint
from langchain_community.embeddings import HuggingFaceHubEmbeddings
from langchain.memory import ConversationBufferMemory

from database.database import Database
from constants.constants import (
    debug_mode,
    DEFAULT_TOKENS_PER_MINUTE,
    OPEN_AI_EMBEDDINGS_COST_PER_THOUSAND_TOKENS,
    DEFAULT_OPEN_AI_TEMPERATURE,
)
from constants.constants import (
    DEFAULT_OPEN_AI_MODEL,
    OPEN_AI_MODEL_TYPE_NAME,
    HUGGING_FACE_MIXTRAL_MODEL,
    HUGGING_FACE_LLAMA_MODEL,
)
from constants.constants import (
    HUGGING_FACE_MIXTRAL_MODEL_ID,
    HUGGING_FACE_LLAMA_MODEL,
    HUGGING_FACE_LLAMA_MODEL_ID,
)
from constants.constants import (
    HUGGING_FACE_FALCON_MODEL,
    HUGGING_FACE_FALCON_MODEL_ID,
    HUGGING_FACE_TEXT_EMBEDDING_MODEL_NAME,
)
from utils.utils import Utils
from loaders.berrydb_json_loader import BerryDBJSONLoader


class Embeddings:
    __berry_db_api_key: str
    __open_ai_api_key: str
    __bucket_name: str
    __database_id: str
    __database_name: str
    __embedding_function: str
    __limit: str
    __tokens_per_minute: int
    __model: str
    __system_prompt: str

    session_models: dict = {}
    session_chains: dict = {}

    def __init__(
        self,
        database_id: int,
        database_name: str,
        bucket_name: str,
        berry_db_api_key: str,
        open_ai_api_key: str,
        limit: int = None,
        embedding_function=None,
        tokens_per_minute: int = None,
        model: str = None,
        system_prompt: str = None,
        session_id: str = None,
    ):
        if not database_id:
            Utils.print_error_and_exit("Database cannot be None")
        if not database_name:
            Utils.print_error_and_exit("Database name cannot be None")
        if not bucket_name:
            Utils.print_error_and_exit("Bucket name cannot be None")
        if not berry_db_api_key:
            Utils.print_error_and_exit("API Key cannot be None")
        if not open_ai_api_key:
            Utils.print_error_and_exit("OpenAI API Key cannot be None")
        if limit and not (isinstance(limit, int) and int(limit) > 0):
            Utils.print_error_and_exit("Limit should be empty or a positive integer")
        if tokens_per_minute and isinstance(tokens_per_minute, int):
            tokens_per_minute = int(tokens_per_minute)
            if not tokens_per_minute > 0:
                Utils.print_error_and_exit(
                    "Tokens per minute should be empty or a positive integer"
                )
            if tokens_per_minute < DEFAULT_TOKENS_PER_MINUTE:
                Utils.print_error_and_exit(
                    f"Tokens per minute cannot be less than {DEFAULT_TOKENS_PER_MINUTE}"
                )

        self.__berry_db_api_key = berry_db_api_key
        self.__open_ai_api_key = open_ai_api_key
        self.__bucket_name = bucket_name
        self.__database_id = database_id
        self.__database_name = database_name
        self.__embedding_function = embedding_function
        self.__limit = int(limit) if limit else None
        self.__tokens_per_minute = (
            tokens_per_minute if tokens_per_minute else DEFAULT_TOKENS_PER_MINUTE
        )
        self.__model = model
        self.__system_prompt = system_prompt
        self.__session_id = session_id

    def embedDb(self):
        if debug_mode:
            print("self.__database_id: ", self.__database_id)
            print("self.__database_name: ", self.__database_name)
            print("self.__bucket_name: ", self.__bucket_name)
            print("self.__tokens_per_minute: ", self.__tokens_per_minute)
            print("self.__model: ", self.__model)
            print("self.__system_prompt: ", self.__system_prompt)
            # print(f"Checking if you have access to {self.__database_name}")

        documents = []
        records = self.__get_all_records_for_database()
        records = [item["BerryDb"] for item in records]
        if self.__limit:
            records = records[: self.__limit]
        if debug_mode:
            print(f"Fetched {len(records)} records")
        if len(records) <= 0:
            Utils.print_error_and_exit("Database does not have any records")

        documents = self.__load_JSON(records)

        tokens, embedding_cost = self.__calculate_embedding_cost(documents)
        if debug_mode:
            print(f"Embedding Cost: ${embedding_cost:.4f}")
            print(f"Number of chunks: {len(documents)}")

        return self.__embed_docs(documents, True, self.__embedding_function)

    def __get_chat_model(self, temperature):
        try:
            memory = None
            if self.__session_id:
                print(
                    "*************************Finding llm_model for session with id: ",
                    self.__session_id,
                )
                """ llm_model = (
                    Embeddings.session_models[self.__session_id]
                    if self.__session_id in Embeddings.session_models
                    else None
                ) """
                memory = (
                    Embeddings.session_models[self.__session_id]
                    if self.__session_id in Embeddings.session_models
                    else None
                )
                if memory:
                    print(
                        "*************************Found memory for: ",
                        self.__session_id,
                        " memory: ",
                        memory,
                    )
                """ if llm_model:
                    print(
                        "*************************Found llm_model for: ",
                        self.__session_id,
                    )
                    return llm_model """
            """ print(
                "*************************Couldn't find llm_model for: ",
                self.__session_id,
            ) """
            if self.__model == HUGGING_FACE_MIXTRAL_MODEL_ID:
                # max_length=128,
                llm_model = HuggingFaceEndpoint(
                    repo_id=HUGGING_FACE_MIXTRAL_MODEL_ID,
                    temperature=temperature,
                    model_kwargs={"trust_remote_code": True},
                    huggingfacehub_api_token=self.__open_ai_api_key,
                )
                # return HuggingFaceHub(HUGGING_FACE_MIXTRAL_MODEL_ID)
            elif self.__model == HUGGING_FACE_LLAMA_MODEL_ID:
                llm_model = HuggingFaceEndpoint(
                    repo_id=HUGGING_FACE_LLAMA_MODEL_ID,
                    temperature=temperature,
                    model_kwargs={"trust_remote_code": True},
                    huggingfacehub_api_token=self.__open_ai_api_key,
                )
                # return HuggingFaceHub(HUGGING_FACE_LLAMA_MODEL_ID)
            elif self.__model == HUGGING_FACE_FALCON_MODEL_ID:
                llm_model = HuggingFaceEndpoint(
                    repo_id=HUGGING_FACE_FALCON_MODEL_ID,
                    temperature=temperature,
                    model_kwargs={"trust_remote_code": True},
                    huggingfacehub_api_token=self.__open_ai_api_key,
                )
                # return HuggingFaceHub(HUGGING_FACE_LLAMA_MODEL_ID)
            else:
                self.__model = self.__model or DEFAULT_OPEN_AI_MODEL
                llm_model = ChatOpenAI(
                    model=self.__model,
                    temperature=temperature or DEFAULT_OPEN_AI_TEMPERATURE,
                    openai_api_key=self.__open_ai_api_key,
                )
            """ if self.__session_id:
                Embeddings.session_models[self.__session_id] = llm_model """
            if self.__session_id:
                if not memory:
                    memory = ConversationBufferMemory(
                        memory_key="history", input_key="question", return_messages=True
                    )
                    Embeddings.session_models[self.__session_id] = memory
            return llm_model
            # else:
            #     Utils.print_error_and_exit(f"Model named {self.__model} not found")
        except Exception as e:
            raise e
            # Utils.print_error_and_exit(
            #     f"Unable to find model named {self.__model}, reason: {e['error']['message'] if (('error' in e) and ('message' in e['error'])) else e}"
            # )

    def ask_and_get_answers(
        self,
        q,
        k=None,
        temperature=None,
    ):
        """llm = ChatOpenAI(
            model="gpt-3.5-turbo", temperature=temperature, openai_api_key=self.__open_ai_api_key
        )"""
        chain = None
        prompt_template = (
            "System: " + self.__system_prompt if self.__system_prompt else ""
        )
        llm = self.__get_chat_model(temperature)
        v_store = self.__get_vector_store()
        retriever = v_store.as_retriever(search_kwargs={"k": k or 3})

        if self.__session_id:
            chain = (
                Embeddings.session_chains[self.__session_id]
                if self.__session_id in Embeddings.session_chains
                else None
            )
            """prompt_template +=

            {context}

            Question: {question}
            """
            # if memory:
            prompt_template += """
            Use the following context (delimited by <ctx></ctx>) and the chat history (delimited by <hs></hs>) to answer the question:
            ------
            <ctx>
            {context}
            </ctx>
            ------
            <hs>
            {history}
            </hs>
            ------
            {question}
            Answer:
            """
            # else:
            #     prompt_template += """
            #     Use the following context (delimited by <ctx></ctx>) to answer the question:
            #     ------
            #     <ctx>
            #     {context}
            #     </ctx>
            #     ------
            #     {question}
            #     Answer:
            #     """
            prompt_with_question = PromptTemplate(
                template=prompt_template,
                input_variables=(["history", "context", "question"]),
            )
            chain_type_kwargs = {
                "prompt": prompt_with_question,
                "verbose": True,
            }
            # if memory:
            #     chain_type_kwargs["memory"] = memory
            # chain_type_kwargs = {"prompt": final_prompt}
        if not chain:
            prompt_template += """
                Use the following context (delimited by <ctx></ctx>) to answer the question:
                ------
                <ctx>
                {context}
                </ctx>
                ------
                {question}
                Answer:
                """
            prompt_with_question = PromptTemplate(
                template=prompt_template,
                input_variables=(["context", "question"]),
            )
            chain_type_kwargs = {
                "prompt": prompt_with_question,
                "verbose": True,
            }
            chain = RetrievalQA.from_chain_type(
                llm=llm,
                chain_type="stuff",
                retriever=retriever,
                chain_type_kwargs=chain_type_kwargs,
            )
            if self.__session_id:
                Embeddings.session_chains[self.__session_id] = chain
        answer = chain.run(q)
        if debug_mode:
            print("answer: ", answer)
        return answer

    def __get_embeddings_from_docs(self, open_ai_api_key):
        if (
            self.__model == HUGGING_FACE_MIXTRAL_MODEL_ID
            or self.__model == HUGGING_FACE_LLAMA_MODEL_ID
            or self.__model == HUGGING_FACE_FALCON_MODEL_ID
        ):
            return HuggingFaceHubEmbeddings(
                model=HUGGING_FACE_TEXT_EMBEDDING_MODEL_NAME,
                task="feature-extraction",
                huggingfacehub_api_token=open_ai_api_key,
            )
        return OpenAIEmbeddings(openai_api_key=open_ai_api_key)

    def __get_vector_store(self):
        if not self.is_embedded():
            Utils.print_error_and_exit(
                f"Database with name {self.__database_name} is not embedded. Please embed the database and try again."
            )
        records = self.__get_all_records_for_database()
        records = [item["BerryDb"] for item in records]
        docs = self.__load_JSON(records)
        vector_store = self.__embed_docs(docs)
        return vector_store

    def __embed_docs(
        self,
        documents,
        generate_embeddings=False,
        embedding_function=None,
    ):
        if embedding_function and not callable(embedding_function):
            Utils.print_error_and_exit("Embedding Function must be a function or None")

        texts = [d.page_content for d in documents]
        metadatas = [d.metadata for d in documents]

        if embedding_function:
            return embedding_function(documents)

        embedding = self.__get_embeddings_from_docs(self.__open_ai_api_key)

        if generate_embeddings:
            # self.get_embeddings_from_docs(generate_embeddings, embedding_function)

            # embeddings = embedding.embed_documents(texts)
            embeddings = self.__embed_texts_with_rate_limit(
                embedding, texts, self.__tokens_per_minute
            )
            index = faiss.IndexFlatIP(len(embeddings[0]))
            vector = np.array(embeddings, dtype=np.float32)
            index.add(vector)
        else:
            try:
                embeddings_file_path = self.__get_embeddings_path(
                    self.__database_id, self.__database_name
                )
                if debug_mode:
                    print("embeddings_file_path: ", embeddings_file_path)
                vector = np.load(embeddings_file_path)
                index = faiss.IndexFlatIP(len(vector[0]))
                index.add(vector)
            except Exception as e:
                # raise e
                Utils.print_error_and_exit(
                    f"Database with name {self.__database_name} is not embedded. Please embed the database and try again."
                )

        documents = []

        for i, text in enumerate(texts):
            metadata = metadatas[i] if metadatas else {}
            documents.append(Document(page_content=text, metadata=metadata))
        index_to_docstore_id = {
            i: doc.metadata["objectId"] for i, doc in enumerate(documents)
        }
        if debug_mode:
            print("index to doctore map")
            # print(index_to_docstore_id)
        docstore = InMemoryDocstore(
            {index_to_docstore_id[i]: doc for i, doc in enumerate(documents)}
        )

        faiss_vector_store = FAISS(
            embedding.embed_query, index, docstore, index_to_docstore_id
        )

        if generate_embeddings:
            file_path = self.__get_embeddings_path(
                self.__database_id, self.__database_name
            )
            doc_store_id_to_index = {
                index_to_docstore_id[i]: i for i in range(len(index_to_docstore_id))
            }
            """ if debug_mode:
                print(doc_store_id_to_index) """
            self.__persist_embeddings(file_path, vector)
            self.__store_embedding_data(file_path, doc_store_id_to_index)

        return faiss_vector_store

    def __embed_texts_with_rate_limit(self, embedding, texts, tokens_per_minute):
        time_limit_in_seconds = 70
        chunked_texts = []
        total_tokens = 0
        timer_start = time.time()
        enc = tiktoken.encoding_for_model("text-embedding-ada-002")

        text_embeddings = []
        print(f"A total of {len(texts)} texts received for embedding\n\n")
        for text in texts:
            # print(f"text: {text}")
            # tokens = enc.count_tokens(text)
            # tokens = len(enc.encode(text))
            tokens = sum([len(enc.encode(words)) for words in text])
            # print(f"tokens: {tokens}")
            if (
                total_tokens + tokens > tokens_per_minute
                and time.time() - timer_start < time_limit_in_seconds
            ):
                remaining_time = max(
                    0, time_limit_in_seconds - (time.time() - timer_start)
                )
                if debug_mode:
                    print(f"total_tokens: {total_tokens}")
                    print(f"tokens: {tokens}")
                    print(f"remaining_time: {remaining_time}")
                    print(f"Embedding {total_tokens} tokens\n")

                text_embeddings.extend(embedding.embed_documents(chunked_texts))
                print(
                    f"Embedded a total of {len(text_embeddings)} of {len(texts)}: ({int(len(text_embeddings) / len(texts) * 100)}%)\n"
                )
                # print(f"Total embeddings length: {len(text_embeddings)}\n")
                chunked_texts = []

                if debug_mode:
                    print(f"Sleeping for {remaining_time} seconds\n")
                time.sleep(remaining_time)
                timer_start = time.time()
                total_tokens = 0

            chunked_texts.append(text)
            total_tokens += tokens

        if chunked_texts:
            print(f"Embedding {len(chunked_texts)} remaining tokens\n")
            text_embeddings.extend(embedding.embed_documents(chunked_texts))
            print(
                f"Embedded a total of {len(text_embeddings)} of {len(texts)}: ({int(len(text_embeddings) / len(texts)) * 100}%)\n"
            )

        print(f"Embedded a total of {len(chunked_texts)} tokens\n\n\n")
        return text_embeddings

    def __get_embeddings_path(self, database_id, database_name):
        embeddings_file_path_prefix = os.environ.get(
            "EMBEDDINGS_FILE_PATH_PREFIX", "embeddings/"
        )
        if debug_mode:
            print("embeddings_file_path_prefix: ", embeddings_file_path_prefix)
        if embeddings_file_path_prefix and not os.path.exists(
            embeddings_file_path_prefix
        ):
            os.makedirs(embeddings_file_path_prefix)
        return f"{embeddings_file_path_prefix if embeddings_file_path_prefix else ''}{str(database_id)}_{database_name}_embeddings.npy"

    def __persist_embeddings(self, file_path, embeddings):
        # Convert embeddings to a NumPy array and persist to file system
        embeddings_array = np.array(embeddings)
        np.save(file_path, embeddings_array)

    def __store_embedding_data(self, file_path, mappings):
        data = {
            "databaseId": self.__database_id,
            "path": file_path,
            "mappings": mappings,
        }
        doc = {"id": str(self.__database_id) + "_" + self.__database_name, "data": data}

        self.__upsert_file_path_data_into_vector_bucket(doc)

    def __calculate_embedding_cost(self, texts):
        enc = tiktoken.encoding_for_model("text-embedding-ada-002")
        total_tokens = sum([len(enc.encode(page.page_content)) for page in texts])
        return (
            total_tokens,
            total_tokens / 1000 * OPEN_AI_EMBEDDINGS_COST_PER_THOUSAND_TOKENS,
        )
        # return len(texts), 0

    def __get_all_records_for_database(self):
        db = Database(
            self.__berry_db_api_key,
            self.__bucket_name,
            self.__database_id,
            self.__database_name,
        )
        records = db.get_all_documents()
        if debug_mode:
            print(f"A total of {len(records)} fetched")
            # print(records)
        return records

    def __load_JSON(self, records):
        # loader = BerryDBJSONCSVLoader(json_data=records, text_content=False)
        loader = BerryDBJSONLoader(
            json_data=records, jq_schema=".[]", text_content=False
        )
        return loader.load()

    def __upsert_file_path_data_into_vector_bucket(self, data):
        berryDb_vectors_bkt = Database(
            self.__berry_db_api_key,
            "Vectors",
            self.__database_id,
            self.__database_name,
        )
        berryDb_vectors_bkt.upsert(data)

    def is_embedded(self):
        berryDb_vectors = Database(
            self.__berry_db_api_key, "Vectors", self.__database_id, self.__database_name
        )
        # query = f'Select * from Vectors where databaseId = {db_id}'
        query = f"SELECT COUNT(*) AS count FROM `Vectors` WHERE databaseId = '{self.__database_id}'"
        result = berryDb_vectors.query(query)
        # if debug_mode:
        # print("result: ", result)
        try:
            count = result[0]["count"]
        except:
            count = 0
        return bool(count)

    def delete_embeddings(self):
        if debug_mode:
            print("self.__database_id: ", self.__database_id)
            print("self.__database_name: ", self.__database_name)
            print("self.__database_name: ", self.__berry_db_api_key)

        self.__delete_embeddings_file()

    def __delete_embeddings_file(self):
        file_path = self.__get_embeddings_path(self.__database_id, self.__database_name)
        try:
            os.remove(file_path)
        except OSError as e:
            print(
                "Error: Failed to delete embeddings file. Database may not have been embedded."
            )


import requests
import json
from utils.utils import Utils
from urllib.parse import urlencode
from label_studio_sdk import Client
from langchain_openai import ChatOpenAI
from database.database import Database
from constants.constants import (
    get_database_list_by_api_key_url,
    get_database_id_url,
    get_schema_id_url,
    create_schema_url,
)
from constants.constants import (
    create_database_url,
    create_label_studio_project_url,
    setup_label_config_url,
    delete_database_url,
)
from constants.constants import (
    reimport_label_studio_project_url,
    connect_project_to_ml_url,
    LABEL_STUDIO_BASE_URL,
)
from constants.constants import create_annotations_url, create_predictions_url
from constants.constants import DEFAULT_BUCKET, debug_mode


class BerryDB:
    @classmethod
    def connect(
        self,
        api_key: str,
        database_name: str,
        bucket_name: str = DEFAULT_BUCKET,
    ):
        """Function summary

        Args:
            arg1 (str): API Key
            arg2 (str): Database Name
            arg3 (str): Bucket Name

        Returns:
            Database: An instance of the database
        """

        if debug_mode:
            print("api_key: ", api_key)
            print("database_name: ", database_name)
            print("bucket_name: ", bucket_name)
            print("\n\n")

        database_id: int = self.__get_database_id(self, api_key, database_name)

        return Database(api_key, bucket_name, database_id, database_name)

    @classmethod
    def databases(self, api_key: str):
        """Function summary

        Args:
            arg1 (str): API Key

        Returns:
            list: Dict of Databases
        """

        url = get_database_list_by_api_key_url
        params = {"apiKey": api_key}

        if debug_mode:
            print("url:", url)
            print("params:", params)

        try:
            response = requests.get(url, params=params)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            jsonResponse = response.json()
            if debug_mode:
                print("databases result ", jsonResponse)
            if (
                "database" in jsonResponse
                and "responseList" in jsonResponse["database"]
            ):
                databaseNames = {}
                # print("\nDatabases:")
                for db in jsonResponse["database"]["responseList"]:
                    name = db["name"] if "name" in db else ""
                    schemaName = db["schemaName"] if "schemaName" in db else ""
                    description = db["description"] if "description" in db else ""
                    dbId = db["id"] if "id" in db else ""
                    schemaId = db["schemaId"] if "schemaId" in db else ""
                    isContentType = db["contentType"] if "contentType" in db else ""
                    databaseNames[name] = {
                        "id": dbId,
                        "schemaId": schemaId,
                        "schemaName": schemaName,
                        "description": description,
                        "isContentType": isContentType,
                    }
                    # print(name + " : " + str(databaseNames[name]))
                # print("\n")
                return databaseNames
            return {}
        except Exception as e:
            print("Failed to fetch databases: {}".format(str(e)))
            return {}

    @classmethod
    def create_schema(self, api_key, schema_name, schema_desc, schema_details):
        """Function summary

        Args:
            arg1 (str): API Key
            arg3 (str): Schema Name
            arg4 (str): Schema Description
            arg5 (str): Schema Details
        Returns:
            str:  Connection to the database
        """
        if debug_mode:
            print("schema_details: ", schema_details)

        url = create_schema_url
        params = {"apiKey": api_key}

        payload = json.dumps(
            {
                "name": schema_name,
                "description": schema_desc,
                "details": schema_details,
            }
        )
        headers = {
            "Authorization": f"Token {api_key}",
            "Content-Type": "application/json",
        }

        if debug_mode:
            print("url:", url)
            print("payload:", payload)
            # print("headers:", headers)

        try:
            response = requests.post(url, params=params, data=payload, headers=headers)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Schema creation result ", response.json())
            json_res = json.loads(response.text)
            if json_res["responseList"][0]["schemaId"]:
                print(
                    json_res["message"]
                    if json_res["message"]
                    else "Schema created successfully"
                )
                res = json_res["responseList"][0]
                res["schema_name"] = schema_name
                return res
            else:
                Utils.handleApiCallFailure(
                    "Schema creation failed, please try again later.",
                    response.status_code,
                )
        except Exception as e:
            print("Failed to create the Schema: {}".format(str(e)))
            return None

    @classmethod
    def create_database(
        self, api_key, database_name, schema_name, bucket_name=DEFAULT_BUCKET
    ):
        """Function summary

        Args:
            arg1 (str): API Key
            arg2 (str): Database Name
            arg3 (str): Schema Name

        Returns:
            Database: An instance of the newly created database, if database with name already exists, prints a failure message and returns None
        """
        schema_id = None
        user_id = None
        try:
            schema_id, user_id = self.__get_schema_id_by_name(api_key, schema_name)
        except Exception as e:
            pass
        if debug_mode:
            print("schema_id :", schema_id)
            print("user_id: ", user_id)
        if not (schema_id and user_id):
            return

        url = create_database_url
        payload = {
            "schemaId": str(schema_id),
            "userId": user_id,
            "databaseName": database_name,
        }
        # headers = Utils.get_headers(api_key, "multipart/form-data")

        if debug_mode:
            print("url:", url)
            print("payload:", payload)
            # print("headers:", headers)

        try:
            session = requests.session()
            response = session.post(url, data=payload)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Database creation result ", response.json())
            json_res = json.loads(response.text)
            if json_res["databaseId"]:
                print(
                    json_res["message"]
                    if json_res["message"]
                    else "Database created successfully"
                )
                return Database(
                    api_key, bucket_name, json_res["databaseId"], database_name
                )
            else:
                Utils.handleApiCallFailure(
                    "Database creation failed, please try again later.",
                    response.status_code,
                )
        except Exception as e:
            print("Failed to create the database: {}".format(str(e)))
            return None

    @classmethod
    def delete_database(self, api_key, database_name):
        """Function summary

        Args:
            arg1 (str): API Key
            arg2 (str): Database Name

        Returns:
            str:  Connection to the database
        """
        schema_id = None
        user_id = None
        database = self.__get_database(self, api_key, database_name)
        print("database :", database)

        schema_id = None
        user_id = None
        try:
            schema_id, user_id = self.__get_schema_id_by_name(
                api_key, database["schemaName"]
            )
        except Exception as e:
            pass
        if debug_mode:
            print("schema_id :", schema_id)
            print("user_id: ", user_id)
        if not (schema_id and user_id):
            return

        url = delete_database_url
        params = {"databaseId": database["id"], "userId": user_id}
        headers = {
            "Authorization": f"Token {api_key}",
            "Content-Type": "application/json",
        }

        if debug_mode:
            print("url:", url)
            print("params:", params)

        try:
            response = requests.delete(url, params=params, headers=headers)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Database deletion result ", response.json())
            json_res = json.loads(response.text)
            return json_res["message"]
        except Exception as e:
            print("Failed to delete the database: {}".format(str(e)))
            return None

    @classmethod
    def create_project(
        self, annotations_api_key, project_name, project_description=""
    ):
        """Function summary

        Args:
            arg1 (str): API Key
            arg2 (str): Project Name
            arg3 (str): Project Description (Optional)

        Returns:
            str:  Success/failure message
        """

        url = create_label_studio_project_url
        payload = {"title": project_name, "description": project_description}
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Token {annotations_api_key}",
        }

        if debug_mode:
            print("url:", url)
            print("payload:", payload)

        try:
            response = requests.post(url, data=json.dumps(payload), headers=headers)
            if response.status_code != 201:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Create LS project result ", response.json())
            # json_res = json.loads(response.text)
            print("Project created successfully")
            return response.json()
        except Exception as e:
            print(f"Failed to create your project: {str(e)}")
            return None

    @classmethod
    def setup_label_config(self, annotations_api_key, project_id, label_config):
        """Function summary

        Args:
            arg1 (str): Annotations API Key
            arg2 (int): Project ID
            arg3 (str): Label Config

        Returns:
            str:  Success/failure message
        """

        url = setup_label_config_url.format(project_id)
        payload = json.dumps({"label_config": label_config})
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Token {annotations_api_key}",
        }

        if debug_mode:
            print("url:", url)
            print("payload:", payload)

        try:
            response = requests.patch(url, data=payload, headers=headers)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Setup config result ", response.json())
            # json_res = json.loads(response.text)
            return response.json()
        except Exception as e:
            print(f"Failed to setup config: {str(e)}")
            return None

    @classmethod
    def populate_project(self, annotations_api_key, project_id, database_id):
        """Function summary

        Args:
            arg1 (str): Annotations API Key
            arg2 (int): Annotations project ID
            arg3 (int): Database ID

        Returns:
            json: Project details
        """

        try:
            reimport_url = reimport_label_studio_project_url.format(project_id)
            headers = {
                "Content-Type": "application/json",
                "Authorization": "Token {}".format(annotations_api_key),
            }
            reimport_data = {
                "file_upload_ids": [],
                "files_as_tasks_list": False,
                "database_id": database_id,
            }

            if debug_mode:
                print("url:", reimport_url)
                print("payload:", reimport_data)

            reimport_response = requests.post(
                reimport_url, data=json.dumps(reimport_data), headers=headers
            )
            if reimport_response.status_code != 201:
                Utils.handleApiCallFailure(
                    reimport_response.json(), reimport_response.status_code
                )
            if debug_mode:
                print("Populate project result: ", reimport_response.json())
            print("Project populated successfully")
            return reimport_response.json()
        except Exception as e:
            print(f"Failed to populate your project: {str(e)}")
            if debug_mode:
                raise e
            return None

    @classmethod
    def connect_project_to_ml(self, annotations_api_key, project_id, ml_url, ml_title="ML Model"):
        """Function summary

        Args:
            arg1 (str): Annotations API Key
            arg2 (int): Annotations project ID
            arg3 (str): ML Backend URL you want to connect the project to
            arg4 (str): A title for the connection (Optional)

        Returns:
            None
        """
        # TODO: Check for existing connected ML models and use the PATCH method to update the existing ML model connection
        try:
            headers = {
                "Content-Type": "application/json",
                "Authorization": "Token {}".format(annotations_api_key),
            }
            ml_payload = {
                "project": project_id,
                "title": ml_title or "ML Model",
                "url": ml_url,
                "is_interactive": False,
            }
            ml_connect_response = requests.post(
                connect_project_to_ml_url, data=json.dumps(ml_payload), headers=headers
            )
            if ml_connect_response.status_code != 201:
                    Utils.handleApiCallFailure(
                        ml_connect_response.json(), ml_connect_response.status_code
                    )
            if debug_mode:
                    print("Connect project to ML result: ", ml_connect_response.json())
            print("Project connected to ML Model successfully")
        except Exception as e:
            print(f"Failed to Connect project to BerryDB ML backend: {str(e)}")
            return None

    @classmethod
    def create_prediction(self, annotations_api_key, project_id, task_id, prediction):
        """Function summary

        Args:
            arg1 (str): Annotations API Key
            arg2 (int): Annotations project ID
            arg3 (int): Task ID in the project
            arg4 (dict): The prediction dict to add to the task (Optional)

        Returns:
            None
        """
        try:
            url = create_predictions_url.format(task_id)

            if type(prediction) != dict:
                print("Error: prediction has to be dict type")
                return
            prediction["project"] = project_id
            prediction["task"] = task_id

            payload = json.dumps(prediction)
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Token {annotations_api_key}",
            }

            if debug_mode:
                print("url:", url)
                print("payload:", payload)

            response = requests.post(url, data=payload, headers=headers, verify=False)
            print("response:", response)
            if response.status_code != 201:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Setup config result ", response.json())
            return response.json()
        except Exception as e:
            print(f"Failed to create prediction: {str(e)}")
            return None

    @classmethod
    def create_annotation(self, annotations_api_key, project_id, task_id, annotation):
        """Function summary

        Args:
            arg1 (str): Annotations API Key
            arg2 (int): Annotations project ID
            arg3 (int): Task ID in the project
            arg4 (dict): The annotation dict to add to the task (Optional)

        Returns:
            None
        """
        try:
            url = create_annotations_url.format(task_id, project_id)

            if type(annotation) != dict:
                print("Error: annotations has to be dict type")
                return
            annotation["project"] = project_id

            payload = json.dumps(annotation)
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Token {annotations_api_key}",
            }

            if debug_mode:
                print("url:", url)
                print("payload:", payload)

            response = requests.post(url, data=payload, headers=headers, verify=False)
            if response.status_code != 201:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Setup config result ", response.json())
            return response.json()
        except Exception as e:
            print(f"Failed to create annotations: {str(e)}")
            return None

    """ @classmethod
    def __get_project(self, annotations_api_key, project_id):
        try:
            if debug_mode:
                print(annotations_api_key)
                print(project_id)
            ls = Client(url=label_studio_base_url, api_key=annotations_api_key)
            project = ls.get_project(id=project_id)
            if not project:
                print(f"Failed to get project")
            if debug_mode:
                print("Fetched project result: ",  project)
            return project
        except Exception as e:
            print(f"Failed to get project: {str(e)}")
            raise Exception(e)
            return None """

    @classmethod
    def get_task_ids(self, annotations_api_key, project_id):
        """Function summary

        Args:
            arg1 (str): Annotations API Key
            arg2 (int): Annotations project ID

        Returns:
            list: List of task ids in the project
        """
        ls = Client(url=LABEL_STUDIO_BASE_URL, api_key=annotations_api_key)
        project = ls.get_project(id=project_id)
        return project.get_tasks(only_ids=True)

    @classmethod
    def get_task_by_id(self, annotations_api_key, project_id, task_id):
        ls = Client(url=LABEL_STUDIO_BASE_URL, api_key=annotations_api_key)
        project = ls.get_project(id=project_id)
        return project.get_task(task_id)

    @classmethod
    def get_tasks(self, annotations_api_key, project_id):
        ls = Client(url=LABEL_STUDIO_BASE_URL, api_key=annotations_api_key)
        project = ls.get_project(id=project_id)
        return project.get_tasks()

    @classmethod
    def get_answer_for_prompt(
        self,
        open_ai_api_key,
        prompt,
        question,
        temperature=None,
        open_ai_model=None,
    ):
        print("Prompt: " + prompt)
        print("Question: " + question)
        llm = ChatOpenAI(
            model=open_ai_model or "gpt-3.5-turbo",
            temperature=temperature or 0.5,
            api_key=open_ai_api_key,
        )
        messages = [
            {"role": "system", "content": prompt},
            {"role": "user", "content": question},
        ]

        try:
            llm_response = llm.invoke(messages)
            llm_response = llm_response.content
            print("Answer: " + str(llm_response))
            return llm_response
        except Exception as e:
            print(e)

    @classmethod
    def __get_schema_id_by_name(self, api_key: str, schema_name: str) -> (int, int):
        """Function summary

        Args:
            arg1 (str): API Key
            arg2 (str): Schema Name

        Returns:
            (int : Schema ID, int : User ID)
        """

        url = get_schema_id_url
        params = {"apiKey": api_key, "schemaName": schema_name}

        if debug_mode:
            print("url:", url)
            print("params:", params)

        try:
            response = requests.get(url, params=params)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Get schema id by name result ", response.json())
            json_res = json.loads(response.text)
            if json_res.get("schema", None):
                return (
                    json_res["schema"].get("id", None),
                    json_res["schema"].get("userId", None),
                )
            return
        except Exception as e:
            # err_msg = "Either the schema does not exist or does not belong to you."
            print("Failed to fetch your schema: {}".format(str(e)))
            return None

    def __get_database_id(self, api_key: str, database_name: str) -> int:
        """Function summary

        Args:
            arg1 (str): API Key
            arg2 (str): Database Name

        Returns:
            int : Database ID
        """

        url = get_database_id_url
        params = {"apiKey": api_key, "databaseName": database_name}

        if debug_mode:
            print("url:", url)
            print("params:", params)

        try:
            response = requests.get(url, params=params)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Get database id by name result: ", response.json())
            json_res = json.loads(response.text)
            if json_res.get("database", None):
                return json_res["database"].get("id", None)
            return
        except Exception as e:
            print("Failed to fetch your database: {}".format(str(e)))
            return None

    def __get_database(self, api_key: str, database_name: str) -> int:
        """Function summary

        Args:
            arg1 (str): API Key
            arg2 (str): Database Name

        Returns:
            int : Database ID
        """

        url = get_database_id_url
        params = {"apiKey": api_key, "databaseName": database_name}

        if debug_mode:
            print("url:", url)
            print("params:", params)

        try:
            response = requests.get(url, params=params)
            if response.status_code != 200:
                Utils.handleApiCallFailure(response.json(), response.status_code)
            if debug_mode:
                print("Get database id by name result: ", response.json())
            json_res = json.loads(response.text)
            if json_res.get("database", None):
                return json_res["database"]
            return
        except Exception as e:
            print("Failed to fetch your database: {}".format(str(e)))
            return None


if __name__ == "__main__":
    pass


from constants.constants import generic_error_message
import re

class Utils:
  @staticmethod
  def get_headers(api_key: str, content_type: str = "application/json"):
    return {"Content-Type": content_type, "x-api-key": api_key, "Accept": "*/*"}

  @staticmethod
  def handleApiCallFailure(res, status_code):
    if status_code == 401:
      Utils.__print_error_and_exit(
        "You are Unauthorized. Please check your API Key"
      )
    if res.get("detail", None):
      errMsg = res["detail"]
    elif res.get("errorMessage", None):
      errMsg = res["errorMessage"]
    else:
      errMsg = generic_error_message if (res == None or res == "") else res
    raise Exception(errMsg)

  @staticmethod
  def print_error_and_exit(msg=None):
    msg = msg if msg is not None else generic_error_message
    print(msg)
    raise Exception(msg)
    # exit()

  @staticmethod
  def validate_json_path(path):
    """ Validates the JSON path to ensure keys are alphanumeric or underscore """
    invalid_key_pattern = re.compile(r'[^a-zA-Z0-9.\[\]]')

    if invalid_key_pattern.search(path):
        raise ValueError(f"Invalid JSON path: '{path}'. Keys can only contain letters, digits, and arrays like [0].")

    if re.search(r'\[\D+\]', path):
        raise ValueError(f"Invalid array index in path: '{path}'. Array indices must be numeric.")

  @staticmethod
  def create_nested_dict(json_path, input_dict):
    def set_nested_value(d, keys, value):
        """ Helper function to set a nested value in a dictionary """
        for i, key in enumerate(keys[:-1]):
            if isinstance(key, int):
                # Handle lists (convert previous level to a list if needed)
                if not isinstance(d, list):
                    d[keys[i-1]] = []  # Convert previous dict entry into a list
                    d = d[keys[i-1]]
                # Ensure the list has enough elements to access the desired index
                while len(d) <= key:
                    d.append({})
                d = d[key]  # Move to the list entry
            else:
                # Handle dictionaries
                if key not in d:
                    d[key] = {}
                d = d[key]

        # Handle the final key or index
        if isinstance(keys[-1], int):
            # We set this to the 0th index
            if not isinstance(d, list):
                d[keys[-2]] = []  # Convert to a list if necessary
                d = d[keys[-2]]
            while len(d) <= 0:  # Always target index 0
                d.append({})
            d[0] = {keys[-1]: value}  # Add value to the 0th index
        else:
            d[keys[-1]] = value

    def parse_json_path(path):
        """ Parses the JSON path and returns a list of keys and indices """
        path = re.sub(r'\[(\d+)\]', '[0]', path)  # Replace any array index with 0 for processing
        keys = []
        for part in path.split('.'):
            match = re.findall(r'(\w+)|\[(\d+)\]', part)
            for key, idx in match:
                if key:
                    keys.append(key)
                if idx:
                    keys.append(int(idx))
        return keys

    keys = parse_json_path(json_path)
    result = {}
    set_nested_value(result, keys, input_dict)

    return result



from setuptools import setup, find_packages

VERSION = "1.1.7"

DESCRIPTION = "The database for unstructured data and AI apps"
AUTHOR = "BerryDB"
URL = "https://berrydb.io"

setup(
    name="berrydb",
    version=VERSION,
    description=DESCRIPTION,
    url=URL,
    author=AUTHOR,
    license="Proprietary",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    install_requires=[
        "requests",
        "openai",
        "langchain",
        "langchain_community",
        "langchain_openai",
        "tiktoken",
        "faiss-cpu",
        "jq",
        "unstructured[pdf]",
        "label-studio-sdk==1.0.5",
    ],
    classifiers=[
        "License :: Other/Proprietary License",
    ],
    py_modules=[
        "database",
        "BerryDB",
        "utils",
        "loaders",
        "embeddings",
    ],
    packages=find_packages(),
)



import logging

debug_mode = False
LOGGING_LEVEL = logging.DEBUG

# BerryDB Base URLs
BASE_URL = "https://api.berrydb.io"
ML_BACKEND_BASE_URL = "http://gpt.berrydb.io"
BERRY_GPT_BASE_URL = "https://gpt.berrydb.io"
LABEL_STUDIO_BASE_URL = "https://api.berrydb.io/annotations"

# Profile service endpoints
get_schema_id_url = BASE_URL + "/profile/schema"
get_database_id_url = BASE_URL + "/profile/database"
create_database_url = BASE_URL + "/profile/database"
delete_database_url = BASE_URL + "/profile/database"
create_schema_url = BASE_URL + "/profile/schema"
get_database_list_by_api_key_url = BASE_URL + "/profile/database/list-by-api-key"

# Label Studio endpoints
create_label_studio_project_url = LABEL_STUDIO_BASE_URL + "/api/projects"
setup_label_config_url = LABEL_STUDIO_BASE_URL + "/api/projects/{}"
import_label_studio_project_url = (
    LABEL_STUDIO_BASE_URL + "/api/projects/{}/import?commit_to_project=false"
)
reimport_label_studio_project_url = LABEL_STUDIO_BASE_URL + "/api/projects/{}/reimport"
connect_project_to_ml_url = LABEL_STUDIO_BASE_URL + "/api/ml"
couchbase_config = "cluster_ip=13.126.134.203;cluster_username=Admin;cluster_password=123456;bucket=BerryDb;scope={};collection=bObject"
create_annotations_url = LABEL_STUDIO_BASE_URL + "/api/tasks/{}/annotations?project={}"
create_predictions_url = LABEL_STUDIO_BASE_URL + "/api/predictions"

# Berrydb service endpoints
documents_url = BASE_URL + "/berrydb/documents"
query_url = BASE_URL + "/berrydb/query"
document_by_id_url = BASE_URL + "/berrydb/documents/{}"
bulk_upsert_documents_url = BASE_URL + "/berrydb/documents/bulk"

# ML backend endpoint
transcription_url = ML_BACKEND_BASE_URL + "/transcription"
transcription_yt_url = ML_BACKEND_BASE_URL + "/transcription-yt"
caption_url = ML_BACKEND_BASE_URL + "/caption"

# Berry GPT backend endpoint
extract_pdf_url = BERRY_GPT_BASE_URL + "/extract-pdf"
embed_database_url = BERRY_GPT_BASE_URL + "/chat/embed"
chat_with_database_url = BERRY_GPT_BASE_URL + "/chat"

# Semantic extraction API endpoints
SEMANTICS_PREDICT_URL = BASE_URL + "/profile/semantics/predictions"
SEMANTICS_ANNOTATE_URL = BASE_URL + "/profile/semantics/annotations"

# Semantic extraction types
NER_SE_TYPE = "NER"
MEDICAL_NER_SE_TYPE = "Medical NER"
TEXT_CLASSIFICATION_SE_TYPE = "Text Classification"
TEXT_SUMMARIZATION_SE_TYPE = "Text Summarization"
IMAGE_CLASSIFICATION_SE_TYPE = "Image Classification"
IMAGE_CAPTIONING_SE_TYPE = "Image Captioning"
PNEUMONIA_SE_TYPE = "Pneumonia"
ALZHEIMER_SE_TYPE = "Alzheimer"
FASHION_SE_TYPE = "Fashion"
AUDIO_TRANSCRIPTION_SE_TYPE = "Audio Transcription"
TEXT_CLASSIFICATION_SE_TYPE = "Text Classification"

generic_error_message = "Oops! something went wrong. Please try again later."

# Default variables
DEFAULT_BUCKET = "BerryDb"
DEFAULT_TOKENS_PER_MINUTE = 150000
OPEN_AI_EMBEDDINGS_COST_PER_THOUSAND_TOKENS = 0.0001

# LLM related variables
DEFAULT_OPEN_AI_MODEL = "gpt-3.5-turbo"
DEFAULT_OPEN_AI_TEMPERATURE = 0.5
OPEN_AI_MODEL_TYPE_NAME = "OpenAI"
HUGGING_FACE_MIXTRAL_MODEL = "Mixtral 7B Instruct v0.2"
HUGGING_FACE_MIXTRAL_MODEL_ID = "mistralai/Mistral-7B-Instruct-v0.1"
HUGGING_FACE_LLAMA_MODEL = "LLama 2 7b chat"
HUGGING_FACE_LLAMA_MODEL_ID = "meta-llama/Llama-2-7b-chat"
HUGGING_FACE_FALCON_MODEL = "Falcon 40b"
HUGGING_FACE_FALCON_MODEL_ID = "tiiuae/falcon-40b"
HUGGING_FACE_TEXT_EMBEDDING_MODEL_NAME = "sentence-transformers/all-mpnet-base-v2"
